const transformImports = require('babel-plugin-transform-imports');
const { resolve } = require('path');
const { sync } = require('glob');

const pfMapper = {
    TextVariants: 'Text',
    DropdownPosition: 'dropdownConstants',
    EmptyStateVariant: 'EmptyState',
    TextListItemVariants: 'TextListItem',
    TextListVariants: 'TextList'
};

const FECMapper = {
    SkeletonSize: 'Skeleton',
    PageHeaderTitle: 'PageHeader'
};

const optsMapper = {
    'patternfly-react-core': (mapper = pfMapper) => ({
        transform: (importName) => {
            const files = sync(
                resolve(
                    __dirname,
                    `../../@patternfly/react-core/dist/js/**/${mapper[importName] || importName}.js`
                )
            );
            if (files.length > 0) {
                return files[0].replace(/.*(?=@patternfly)/, '');
            } else {
                throw `File with importName ${importName} does not exist`;
            }
        },
        preventFullImport: false,
        skipDefaultConversion: true
    }),
    'patternfly-react-icons': () => ({
        transform: (importName) =>
            `@patternfly/react-icons/dist/js/icons/${importName
            .split(/(?=[A-Z])/)
            .join('-')
            .toLowerCase()}`,
        preventFullImport: true
    }),
    'rhcs-components': (mapper = FECMapper) => ({
        transform: (importName) =>
            `@redhat-cloud-services/frontend-components/components/cjs/${mapper[importName] || importName}`,
        preventFullImport: true
    }),
    'rhcs-notifications': () => ({
        transform: (importName) =>
            `@redhat-cloud-services/frontend-components-notifications/cjs/${importName}`,
        preventFullImport: true
    })
};

module.exports = function () {
    return {
        visitor: {
            ImportDeclaration: function (path, { opts, ...state }) {
                const { visitor: { ImportDeclaration } } = transformImports();
                Object.entries(opts).forEach(([ key, enablements ]) => {
                    if (enablements !== false && optsMapper[key]) {
                        const importOpts = optsMapper[key](enablements.mapper);
                        ImportDeclaration(path, {
                            ...state,
                            opts: {
                                ...importOpts,
                                ...enablements
                            }
                        });
                    }
                });
            }
        }
    };
};
